use serde::ser::{Serialize, Serializer, SerializeMap};
use std::collections::HashMap;
use std::convert::TryInto; // Import TryInto
use serde_json::{Map, Value as JsonValue}; // Alias serde_json::Value to JsonValue
use zvariant::{OwnedValue, Value as ZValue}; // Alias zvariant::Value to ZValue

pub fn serialize_actions<S>(actions: &[String], serializer: S) -> Result<S::Ok, S::Error>
where
    S: Serializer,
{
    let mut map = Map::new();

    // Actions are in pairs: [id, label, id, label, ...]
    for pair in actions.chunks(2) {
        if let [id, label] = pair {
            map.insert(id.clone(), JsonValue::String(label.clone()));
        }
    }

    map.serialize(serializer)
}

pub fn serialize_hints<S>(
    hints: &HashMap<String, OwnedValue>,
    serializer: S,
) -> Result<S::Ok, S::Error>
where
    S: Serializer,
{
    // Start serializing the map
    let mut map = serializer.serialize_map(Some(hints.len()))?;
    for (key, value) in hints {
        // Serialize each entry as desired
        map.serialize_entry(key, &HintValueSerializer(value))?;
    }
    map.end()
}

/*
impl<'a> Serialize for HintValueSerializer<'a> {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        // Access the signature and value parts of the OwnedValue
        let signature = self.0.value_signature().to_string();
        let value = &self.0;

        // Serialize them as a map with "signature" and "value" fields
        let mut map = serializer.serialize_map(Some(2))?;
        map.serialize_entry("signature", &signature)?;
        map.serialize_entry("value", value)?;
        map.end()
    }
}*/

// A custom struct to handle serialization of OwnedValue
struct HintValueSerializer<'a>(&'a OwnedValue);


impl<'a> Serialize for HintValueSerializer<'a> {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        let signature = self.0.value_signature().to_string();

        // Extract the raw value correctly
        let raw_value = if let Ok(v) = self.0.downcast_ref::<u8>() {
            JsonValue::from(v) // Removed *v
        } else if let Ok(v) = self.0.downcast_ref::<i32>() {
            JsonValue::from(v) // Removed *v
        } else if let Ok(v) = self.0.downcast_ref::<u64>() {
            JsonValue::from(v) // Removed *v
        } else if let Ok(v) = self.0.downcast_ref::<bool>() {
            JsonValue::Bool(v) // Removed *v
        } else if let Ok(v) = self.0.downcast_ref::<String>() {
            JsonValue::String(v.clone())
        } else if let Ok(v) = self.0.downcast_ref::<&str>() {
            JsonValue::String(v.to_string())
        } else {
            JsonValue::Null // Unsupported types fallback to Null
        };

        // Serialize the final structure as a map
        let mut map = serializer.serialize_map(Some(2))?;
        map.serialize_entry("signature", &signature)?;
        map.serialize_entry("value", &raw_value)?;
        map.end()
    }
}
